// Copyright © 2023 ab180. All rights reserved.

using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using UnrealBuildTool;

public class AirbridgeUnreal : ModuleRules
{
#region Airbridge Settings

	class AirbridgeSettings
	{
		public string AppName;
		public string AppToken;
		public int LogLevel;
		public string IOSURIScheme;
		public string AndroidURIScheme;
		public List<string> CustomDomain;
		public long SessionTimeoutSeconds;
		public bool UserInfoHashEnabled;
		public bool LocationCollectionEnabled;
		public bool TrackAirbridgeLinkOnly;
		public bool AutoStartTrackingEnabled;
		public bool FacebookDeferredAppLinkEnabled;
		public int IOSTrackingAuthorizeTimeoutSeconds;
		public string SdkSignatureSecretID;
		public string SdkSignatureSecret;
		public bool TrackInSessionLifeCycleEventEnabled;
		public bool PauseEventTransmitOnBackgroundEnabled;
		public bool ResetEventBufferEnabled;
		public bool SdkEnabled;
		public string AppMarketIdentifier;
		public int EventMaximumBufferCount;
		public double EventMaximumBufferSize;
		public long EventTransmitIntervalSeconds;
		public string FacebookAppId;
		public bool IsHandleAirbridgeDeeplinkOnly;
		public string InAppPurchaseEnvironment;
		public bool CollectTCFDataEnabled;
		
		public override string ToString()
		{
			return new StringBuilder()
				.Append("AirbridgeSettings:\n")
				.AppendFormat("\tAppName: {0}\n", AppName)
				.AppendFormat("\tAppToken: {0}\n", AppToken)
				.AppendFormat("\tLogLevel: {0}\n", LogLevel)
				.AppendFormat("\tIOSURIScheme: {0}\n", IOSURIScheme)
				.AppendFormat("\tAndroidURIScheme: {0}\n", AndroidURIScheme)
				.AppendFormat("\tCustomDomain: {0}\n", string.Join(", ", CustomDomain ?? new List<string>()))
				.AppendFormat("\tSessionTimeoutSeconds: {0}\n", SessionTimeoutSeconds)
				.AppendFormat("\tUserInfoHashEnabled: {0}\n", UserInfoHashEnabled)
				.AppendFormat("\tLocationCollectionEnabled: {0}\n", LocationCollectionEnabled)
				.AppendFormat("\tTrackAirbridgeLinkOnly: {0}\n", TrackAirbridgeLinkOnly)
				.AppendFormat("\tAutoStartTrackingEnabled: {0}\n", AutoStartTrackingEnabled)
				.AppendFormat("\tFacebookDeferredAppLinkEnabled: {0}\n", FacebookDeferredAppLinkEnabled)
				.AppendFormat("\tIOSTrackingAuthorizeTimeoutSeconds: {0}\n", IOSTrackingAuthorizeTimeoutSeconds)
				.AppendFormat("\tSdkSignatureSecretID: {0}\n", SdkSignatureSecretID)
				.AppendFormat("\tSdkSignatureSecret: {0}\n", SdkSignatureSecret)
				.AppendFormat("\tTrackInSessionLifeCycleEventEnabled: {0}\n", TrackInSessionLifeCycleEventEnabled)
				.AppendFormat("\tPauseEventTransmitOnBackgroundEnabled: {0}\n", PauseEventTransmitOnBackgroundEnabled)
				.AppendFormat("\tResetEventBufferEnabled: {0}\n", ResetEventBufferEnabled)
				.AppendFormat("\tSdkEnabled: {0}\n", SdkEnabled)
				.AppendFormat("\tAppMarketIdentifier: {0}\n", AppMarketIdentifier)
				.AppendFormat("\tEventMaximumBufferCount: {0}\n", EventMaximumBufferCount)
				.AppendFormat("\tEventMaximumBufferSize: {0}\n", EventMaximumBufferSize)
				.AppendFormat("\tEventTransmitIntervalSeconds: {0}\n", EventTransmitIntervalSeconds)
				.AppendFormat("\tFacebookAppId: {0}\n", FacebookAppId)
				.AppendFormat("\tIsHandleAirbridgeDeeplinkOnly: {0}\n", IsHandleAirbridgeDeeplinkOnly)
				.AppendFormat("\tInAppPurchaseEnvironment: {0}\n", InAppPurchaseEnvironment)
				.AppendFormat("\tCollectTCFDataEnabled: {0}\n", CollectTCFDataEnabled)
				.ToString();
		}
	}

#endregion

#region Placeholder

public static class Placeholder
{
	public const string AndroidIntentFilter = "<!--{{INTENT_FILTER_PLACEHOLDER}}-->";
 	public const string IosInfoPlist = "<!--{{INFO_PLIST_PLACEHOLDER}}-->";
}

#endregion
	
	public AirbridgeUnreal(ReadOnlyTargetRules Target) : base(Target)
	{
		PCHUsage = ModuleRules.PCHUsageMode.UseExplicitOrSharedPCHs;
		
		ShadowVariableWarningLevel = WarningLevel.Off;
		
		PublicDependencyModuleNames.AddRange(new string[] { "Engine", "Core", "CoreUObject" });
		PrivateDependencyModuleNames.AddRange(new string[] { "Json", });
		
		// Read Airbridge Settings
		AirbridgeSettings airbridgeSettings = new AirbridgeSettings();
		ReadAirbridgeSettingsConfig(ref airbridgeSettings);
		Console.WriteLine(airbridgeSettings.ToString());
		
		if (Target.Platform == UnrealTargetPlatform.Android)
		{
			UpdateAndroidManifest(airbridgeSettings);
			UpdateAndroidAirbridgeSettings(airbridgeSettings);
			
			PrivateIncludePaths.Add("AirbridgeUnreal/Private/Platforms/Android");

			PrivateDependencyModuleNames.AddRange(new string[] { "Launch" });

			string PluginPath = Utils.MakePathRelativeTo(ModuleDirectory, Target.RelativeEnginePath);
			AdditionalPropertiesForReceipt.Add("AndroidPlugin", Path.Combine(PluginPath, "Airbridge_UPL_Android.xml"));
		}
		else if (Target.Platform == UnrealTargetPlatform.IOS)
		{
			UpdateIOSInfoPlist(airbridgeSettings);
			UpdateIOSAppSetting(airbridgeSettings);
			
			PrivateIncludePaths.Add("AirbridgeUnreal/Private/Platforms/iOS");

#if UE_4_26_OR_LATER
            PublicAdditionalFrameworks.Add(
                new Framework(
                    "Airbridge",
                    "../ThirdParty/iOS/airbridge-ios-sdk/Airbridge.embeddedframework.zip",
					null,
					true
                )
            );
#else
            PublicAdditionalFrameworks.Add(
                new Framework(
                    "Airbridge",
                    "../ThirdParty/iOS/airbridge-ios-sdk/Airbridge.embeddedframework.zip"
                )
            );
#endif
            
            PublicFrameworks.AddRange(
                new string[]
                {
                   "AdSupport", "AppTrackingTransparency", "AdServices", "CoreTelephony", "StoreKit", "WebKit" 
                }
            );
            
            string PluginPath = Utils.MakePathRelativeTo(ModuleDirectory, Target.RelativeEnginePath);
            AdditionalPropertiesForReceipt.Add("IOSPlugin", Path.Combine(PluginPath, "Airbridge_UPL_iOS.xml"));
		}
	}
	
	private void ReadAirbridgeSettingsConfig(ref AirbridgeSettings Settings)
	{
		// Read from config
		ConfigHierarchy Ini = ConfigCache.ReadHierarchy(ConfigHierarchyType.Engine, Target.ProjectFile.Directory, Target.Platform);

		string SettingsSection = "/Script/AirbridgeEditor.AirbridgeSettings";
		Ini.GetString(SettingsSection, "AppName", out Settings.AppName);
		Ini.GetString(SettingsSection, "AppToken", out Settings.AppToken);
		Ini.GetAirbridgeLogLevel(SettingsSection, "LogLevel", out Settings.LogLevel);
		Ini.GetString(SettingsSection, "IOSURIScheme", out Settings.IOSURIScheme);
		Ini.GetString(SettingsSection, "AndroidURIScheme", out Settings.AndroidURIScheme);
		Ini.GetArray(SettingsSection, "CustomDomain", out Settings.CustomDomain);
		Ini.GetInt64(SettingsSection, "SessionTimeoutSeconds", out Settings.SessionTimeoutSeconds);
		Ini.GetBool(SettingsSection, "UserInfoHashEnabled", out Settings.UserInfoHashEnabled);
		Ini.GetBool(SettingsSection, "LocationCollectionEnabled", out Settings.LocationCollectionEnabled);
		Ini.GetBool(SettingsSection, "TrackAirbridgeLinkOnly", out Settings.TrackAirbridgeLinkOnly);
		Ini.GetBool(SettingsSection, "AutoStartTrackingEnabled", out Settings.AutoStartTrackingEnabled);
		Ini.GetBool(SettingsSection, "FacebookDeferredAppLinkEnabled", out Settings.FacebookDeferredAppLinkEnabled);
		Ini.GetInt32(SettingsSection, "IOSTrackingAuthorizeTimeoutSeconds", out Settings.IOSTrackingAuthorizeTimeoutSeconds);
		Ini.GetString(SettingsSection, "SdkSignatureSecretID", out Settings.SdkSignatureSecretID);
		Ini.GetString(SettingsSection, "SdkSignatureSecret", out Settings.SdkSignatureSecret);
		Ini.GetBool(SettingsSection, "TrackInSessionLifeCycleEventEnabled", out Settings.TrackInSessionLifeCycleEventEnabled);
		Ini.GetBool(SettingsSection, "PauseEventTransmitOnBackgroundEnabled", out Settings.PauseEventTransmitOnBackgroundEnabled);
		Ini.GetBool(SettingsSection, "ResetEventBufferEnabled", out Settings.ResetEventBufferEnabled);
		Ini.GetBool(SettingsSection, "SdkEnabled", out Settings.SdkEnabled);
		Ini.GetString(SettingsSection, "AppMarketIdentifier", out Settings.AppMarketIdentifier);
		Ini.GetInt32(SettingsSection, "EventMaximumBufferCount", out Settings.EventMaximumBufferCount);
		Ini.TryGetValue(SettingsSection, "EventMaximumBufferSize", out Settings.EventMaximumBufferSize);
		Ini.GetInt64(SettingsSection, "EventTransmitIntervalSeconds", out Settings.EventTransmitIntervalSeconds);
		Ini.GetString(SettingsSection, "FacebookAppId", out Settings.FacebookAppId);
		Ini.GetBool(SettingsSection, "IsHandleAirbridgeDeeplinkOnly", out Settings.IsHandleAirbridgeDeeplinkOnly);
		Ini.GetAirbridgeInAppPurchaseEnvironment(SettingsSection, "InAppPurchaseEnvironment", out Settings.InAppPurchaseEnvironment);
		Ini.GetBool(SettingsSection, "CollectTCFDataEnabled", out Settings.CollectTCFDataEnabled);
	}
	
#region Android Airbridge Settings

	private void UpdateAndroidManifest(AirbridgeSettings Settings)
	{
		string uplSrcPath = Path.Combine(ModuleDirectory, "../AirbridgeEditor/Template/Airbridge_UPL_Android.xml");
		string uplDestPath = Path.Combine(ModuleDirectory, "Airbridge_UPL_Android.xml");

		try
		{
			if (!File.Exists(uplDestPath))
			{
				File.Create(uplDestPath).Dispose();
				Console.WriteLine("Create file : {0}", uplDestPath);
			}

			StringBuilder replacement = new StringBuilder();

			if (!string.IsNullOrEmpty(Settings.AppName))
			{
				replacement
					// YOUR_APP_NAME.airbridge.io
					.Append(GenerateIntentFilter(true, "http", String.Format("{0}.airbridge.io", Settings.AppName)))
					.Append(GenerateIntentFilter(true, "https", String.Format("{0}.airbridge.io", Settings.AppName)))
					// YOUR_APP_NAME.abr.ge
					.Append(GenerateIntentFilter(true, "http", String.Format("{0}.abr.ge", Settings.AppName)))
					.Append(GenerateIntentFilter(true, "https", String.Format("{0}.abr.ge", Settings.AppName)))
					// YOUR_APP_NAME.deeplink.page
					// https://help.airbridge.io/ko/developers/android-sdk#deeplink-page
					.Append(GenerateIntentFilter(true, "http", String.Format("{0}.deeplink.page", Settings.AppName)))
					.Append(GenerateIntentFilter(true, "https", String.Format("{0}.deeplink.page", Settings.AppName)));
			}

			if (!string.IsNullOrEmpty(Settings.AndroidURIScheme))
			{
				// YOUR_APP_URI_SCHEME
				replacement.Append(GenerateIntentFilter(false, Settings.AndroidURIScheme));
			}

			if (Settings.CustomDomain != null)
			{
				foreach (string host in Settings.CustomDomain)
				{
					if (!string.IsNullOrEmpty(host))
					{
						replacement
							.Append(GenerateIntentFilter(true, "http", host))
							.Append(GenerateIntentFilter(true, "https", host));
					}
				}
			}

			// Read source UPL file
			string content = File.ReadAllText(uplSrcPath);

			// Replace the placeholder with the desired string
			content = content.Replace(Placeholder.AndroidIntentFilter, replacement.ToString());

			// Overwrite the updated content to destination UPL file
			File.WriteAllText(uplDestPath, content);

			Console.WriteLine("Updated Airbridge Android UPL file.");
		}
		catch (Exception e)
		{
			Console.WriteLine("Something broken while updating Airbridge Android UPL file : {0}", e.Message);
		}
	}
	
	private string GenerateIntentFilter(bool autoVerify, string scheme, string host = "")
	{
		StringBuilder sb = new StringBuilder();
        
		if (!string.IsNullOrEmpty(scheme))
		{
			sb.AppendFormat("<intent-filter android:autoVerify=\"{0}\">\n", autoVerify.ToString().ToLower())
				.Append("\t<action android:name=\"android.intent.action.VIEW\" />\n")
				.Append("\t<category android:name=\"android.intent.category.DEFAULT\" />\n")
				.Append("\t<category android:name=\"android.intent.category.BROWSABLE\" />\n");
            
			if (!string.IsNullOrEmpty(host))
			{
				sb.AppendFormat("\t<data android:scheme=\"{0}\" android:host=\"{1}\" />\n", scheme, host);
			}
			else
			{
				sb.AppendFormat("\t<data android:scheme=\"{0}\" />\n", scheme);
			}

			sb.Append("</intent-filter>\n");
		}
        
		return sb.ToString();
	}

	private void UpdateAndroidAirbridgeSettings(AirbridgeSettings Settings)
	{
		try
		{
			string path = Path.Combine(ModuleDirectory, "Private/Platforms/Android/Native/AirbridgeSettings.java");
			if (!File.Exists(path))
			{
				File.Create(path).Dispose();
			}

			string content = new StringBuilder()
				.Append("package co.ab180.airbridge.ue;\n")
				.Append("\n")
				.Append("public class AirbridgeSettings {\n")
				.Append("\n")
				.AppendFormat("public static String appName = \"{0}\";\n", Settings.AppName)
				.AppendFormat("public static String appToken = \"{0}\";\n", Settings.AppToken)
				.AppendFormat("public static String sdkSignatureSecretID = \"{0}\";\n", Settings.SdkSignatureSecretID)
				.AppendFormat("public static String sdkSignatureSecret = \"{0}\";\n", Settings.SdkSignatureSecret)
				.AppendFormat("public static int logLevel = {0};\n", Settings.LogLevel)
				// Remove duplicates using Distinct()
				.AppendFormat("public static String customDomain = \"{0}\";\n",
					(Settings.CustomDomain != null) ? String.Join(" ", Settings.CustomDomain.Distinct().ToList()) : "")
				.AppendFormat("public static int sessionTimeoutSeconds = {0};\n", Settings.SessionTimeoutSeconds)
	            .AppendFormat("public static boolean userInfoHashEnabled = {0};\n", Settings.UserInfoHashEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean locationCollectionEnabled = {0};\n", Settings.LocationCollectionEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean trackAirbridgeLinkOnly = {0};\n", Settings.TrackAirbridgeLinkOnly.ToString().ToLower())
	            .AppendFormat("public static boolean autoStartTrackingEnabled = {0};\n", Settings.AutoStartTrackingEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean facebookDeferredAppLinkEnabled = {0};\n", Settings.FacebookDeferredAppLinkEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean trackInSessionLifeCycleEventEnabled = {0};\n", Settings.TrackInSessionLifeCycleEventEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean pauseEventTransmitOnBackgroundEnabled = {0};\n", Settings.PauseEventTransmitOnBackgroundEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean clearEventBufferOnInitializeEnabled = {0};\n", Settings.ResetEventBufferEnabled.ToString().ToLower())
	            .AppendFormat("public static boolean sdkEnabled = {0};\n", Settings.SdkEnabled.ToString().ToLower())
				.AppendFormat("public static String appMarketIdentifier = \"{0}\";\n", Settings.AppMarketIdentifier)
	            .AppendFormat("public static int eventBufferCountLimitInGibibyte = {0};\n", Settings.EventMaximumBufferCount)
	            .AppendFormat("public static double eventBufferSizeLimitInGibibyte = {0};\n", Settings.EventMaximumBufferSize)
	            .AppendFormat("public static long eventTransmitIntervalSeconds = {0};\n", Settings.EventTransmitIntervalSeconds)
	            .AppendFormat("public static String facebookAppId = \"{0}\";\n", Settings.FacebookAppId)
				.AppendFormat("public static boolean isHandleAirbridgeDeeplinkOnly = {0};\n", Settings.IsHandleAirbridgeDeeplinkOnly.ToString().ToLower())
				.AppendFormat("public static String inAppPurchaseEnvironment = \"{0}\";\n", Settings.InAppPurchaseEnvironment)
				.AppendFormat("public static boolean collectTCFDataEnabled = {0};\n", Settings.CollectTCFDataEnabled.ToString().ToLower())
	            .Append("\n}")
				.ToString();

            File.WriteAllText(path, content);
            
			Console.WriteLine("Updated Android Airbridge settings (AirbridgeSettings.java)");
		}
		catch (Exception e)
		{
			Console.WriteLine("Something broken while updating Android Airbridge settings file : {0}", e.Message);
		}
	}

#endregion

#region iOS Airbridge Settings

	private void UpdateIOSInfoPlist(AirbridgeSettings Settings) 
	{
		string uplSrcPath = Path.Combine(ModuleDirectory, "../AirbridgeEditor/Template/Airbridge_UPL_iOS.xml");
		string uplDestPath = Path.Combine(ModuleDirectory, "Airbridge_UPL_iOS.xml");
		
		try
		{
			if (!File.Exists(uplDestPath))
			{
				File.Create(uplDestPath).Dispose();
				Console.WriteLine("Create file : {0}", uplDestPath);
			}
            
			StringBuilder replacement = new StringBuilder();

			replacement.Append(IOS_SetURIScheme(Settings.IOSURIScheme))
				.Append(IOS_SetCustomDomains(Settings.CustomDomain));
            
			// Read source UPL file
			string content = File.ReadAllText(uplSrcPath);

			// Replace the placeholder with the desired string
			content = content.Replace(Placeholder.IosInfoPlist, replacement.ToString());

			// Overwrite the updated content to destination UPL file
			File.WriteAllText(uplDestPath, content);

			Console.WriteLine("Updated Airbridge iOS UPL file.");
		}
		catch (Exception e)
		{
			Console.WriteLine("Something broken while updating Airbridge iOS UPL file : {0}", e.Message);
		}
	}
	
	private string IOS_SetURIScheme(string scheme)
	{
		StringBuilder sb = new StringBuilder();

		if (!string.IsNullOrEmpty(scheme))
		{
			sb.Append("<key>CFBundleURLTypes</key>\n")
				.Append("<array>\n")
				.Append("\t<dict>\n")
				.Append("\t\t<key>CFBundleURLSchemes</key>\n")
				.Append("\t\t<array>\n")
				.AppendFormat("\t\t\t<string>{0}</string>\n", scheme)
				.Append("\t\t</array>\n")
				.Append("\t</dict>\n")
				.Append("</array>\n");
		}

		return sb.ToString();
	}
    
	private string IOS_SetCustomDomains(List<string> customDomains)
	{
		StringBuilder sb = new StringBuilder();

		bool isFirst = true;
        
		if (customDomains != null)
		{
			foreach (string host in customDomains)
			{
				if (!string.IsNullOrEmpty(host))
				{
					if (isFirst)
					{
						sb.Append("<key>co.ab180.airbridge.trackingLink.customDomains</key>\n")
							.Append("<array>\n");
                        
						isFirst = false;
					}
                    
					sb.AppendFormat("\t<string>{0}</string>\n", host);
				}
			}
		}

		if (!isFirst)
		{
			sb.Append("</array>\n");
		}
        
		return sb.ToString();
	}

    private void UpdateIOSAppSetting(AirbridgeSettings Settings)
    {
        try
        {
            string headerPath = Path.Combine(ModuleDirectory, "Private/Platforms/IOS/Native/AUAppSetting.h");
            string implementationPath = Path.Combine(ModuleDirectory, "Private/Platforms/IOS/Native/AUAppSetting.mm");

            // Create header file
            string headerContent = new StringBuilder()
                .Append("#import <Foundation/Foundation.h>\n\n")
                .Append("@interface AUAppSetting : NSObject\n\n")
                .Append("@property (class, nonatomic, strong) NSString *appName;\n")
                .Append("@property (class, nonatomic, strong) NSString *appToken;\n")
                .Append("@property (class, nonatomic, strong) NSString *customDomain;\n")
                .Append("@property (class, nonatomic, strong) NSString *sdkSignatureSecretID;\n")
                .Append("@property (class, nonatomic, strong) NSString *sdkSignatureSecret;\n")
                .Append("@property (class, nonatomic, assign) NSInteger logLevel;\n")
                .Append("@property (class, nonatomic, strong) NSString *appScheme;\n")
                .Append("@property (class, nonatomic, assign) NSInteger sessionTimeoutSeconds;\n")
                .Append("@property (class, nonatomic, assign) BOOL autoStartTrackingEnabled;\n")
                .Append("@property (class, nonatomic, assign) BOOL userInfoHashEnabled;\n")
                .Append("@property (class, nonatomic, assign) BOOL trackAirbridgeLinkOnly;\n")
                .Append("@property (class, nonatomic, assign) BOOL facebookDeferredAppLinkEnabled;\n")
                .Append("@property (class, nonatomic, assign) NSInteger trackingAuthorizeTimeoutSeconds;\n")
                .Append("@property (class, nonatomic, assign) BOOL trackInSessionLifeCycleEventEnabled;\n")
                .Append("@property (class, nonatomic, assign) BOOL pauseEventTransmitOnBackgroundEnabled;\n")
                .Append("@property (class, nonatomic, assign) BOOL clearEventBufferOnInitializeEnabled;\n")
                .Append("@property (class, nonatomic, assign) BOOL sdkEnabled;\n")
                .Append("@property (class, nonatomic, assign) NSInteger eventBufferCountLimitInGibibyte;\n")
                .Append("@property (class, nonatomic, assign) NSInteger eventBufferSizeLimitInGibibyte;\n")
                .Append("@property (class, nonatomic, assign) NSInteger eventTransmitIntervalSeconds;\n")
                .Append("@property (class, nonatomic, assign) BOOL isHandleAirbridgeDeeplinkOnly;\n")
                .Append("@property (class, nonatomic, strong) NSString *inAppPurchaseEnvironment;\n")
                .Append("@property (class, nonatomic, assign) BOOL collectTCFDataEnabled;\n\n")
                .Append("@end\n")
                .ToString();

            File.WriteAllText(headerPath, headerContent);

			// Create implementation file
			string implementationContent = new StringBuilder()
				.Append("#import \"AUAppSetting.h\"\n\n")
				.Append("@implementation AUAppSetting\n\n")

				// Static variable declarations
				.AppendFormat("static NSString *_appName = @\"{0}\";\n", Settings.AppName)
				.AppendFormat("static NSString *_appToken = @\"{0}\";\n", Settings.AppToken)
				.AppendFormat("static NSString *_customDomain = @\"{0}\";\n", String.Join(" ", Settings.CustomDomain.Distinct().ToList()))
				.AppendFormat("static NSString *_sdkSignatureSecretID = @\"{0}\";\n", Settings.SdkSignatureSecretID)
				.AppendFormat("static NSString *_sdkSignatureSecret = @\"{0}\";\n", Settings.SdkSignatureSecret)
				.AppendFormat("static NSInteger _logLevel = {0};\n", Settings.LogLevel)
				.AppendFormat("static NSString *_appScheme = @\"{0}\";\n", Settings.AppName.ToLower())
				.AppendFormat("static NSInteger _sessionTimeoutSeconds = {0};\n", Settings.SessionTimeoutSeconds)
				.AppendFormat("static BOOL _autoStartTrackingEnabled = {0};\n", Settings.AutoStartTrackingEnabled ? "YES" : "NO")
				.AppendFormat("static BOOL _userInfoHashEnabled = {0};\n", Settings.UserInfoHashEnabled ? "YES" : "NO")
				.AppendFormat("static BOOL _trackAirbridgeLinkOnly = {0};\n", Settings.TrackAirbridgeLinkOnly ? "YES" : "NO")
				.AppendFormat("static BOOL _facebookDeferredAppLinkEnabled = {0};\n", Settings.FacebookDeferredAppLinkEnabled ? "YES" : "NO")
				.Append("static NSInteger _trackingAuthorizeTimeoutSeconds = 30;\n") // Default value
				.AppendFormat("static BOOL _trackInSessionLifeCycleEventEnabled = {0};\n", Settings.TrackInSessionLifeCycleEventEnabled ? "YES" : "NO")
				.AppendFormat("static BOOL _pauseEventTransmitOnBackgroundEnabled = {0};\n", Settings.PauseEventTransmitOnBackgroundEnabled ? "YES" : "NO")
				.AppendFormat("static BOOL _clearEventBufferOnInitializeEnabled = {0};\n", Settings.ResetEventBufferEnabled ? "YES" : "NO")
				.AppendFormat("static BOOL _sdkEnabled = {0};\n", Settings.SdkEnabled ? "YES" : "NO")
				.AppendFormat("static NSInteger _eventBufferCountLimitInGibibyte = {0};\n", Settings.EventMaximumBufferCount)
				.AppendFormat("static NSInteger _eventBufferSizeLimitInGibibyte = {0};\n", (int)Settings.EventMaximumBufferSize)
				.AppendFormat("static NSInteger _eventTransmitIntervalSeconds = {0};\n", Settings.EventTransmitIntervalSeconds)
				.AppendFormat("static BOOL _isHandleAirbridgeDeeplinkOnly = {0};\n", Settings.IsHandleAirbridgeDeeplinkOnly ? "YES" : "NO")
				.AppendFormat("static NSString *_inAppPurchaseEnvironment = @\"{0}\";\n", Settings.InAppPurchaseEnvironment)
				.AppendFormat("static BOOL _collectTCFDataEnabled = {0};\n\n", Settings.CollectTCFDataEnabled ? "YES" : "NO")
				
				// Getter and setter methods
				// Getter methods
				.Append("+ (NSString *)appName {\n    return _appName;\n}\n\n")
				.Append("+ (void)setAppName:(NSString *)appName {\n    _appName = [appName copy];\n}\n\n")

				.Append("+ (NSString *)appToken {\n    return _appToken;\n}\n\n")
				.Append("+ (void)setAppToken:(NSString *)appToken {\n    _appToken = [appToken copy];\n}\n\n")

				.Append("+ (NSString *)customDomain {\n    return _customDomain;\n}\n\n")
				.Append("+ (void)setCustomDomain:(NSString *)customDomain {\n    _customDomain = [customDomain copy];\n}\n\n")

				.Append("+ (NSString *)sdkSignatureSecretID {\n    return _sdkSignatureSecretID;\n}\n\n")
				.Append("+ (void)setSdkSignatureSecretID:(NSString *)sdkSignatureSecretID {\n    _sdkSignatureSecretID = [sdkSignatureSecretID copy];\n}\n\n")

				.Append("+ (NSString *)sdkSignatureSecret {\n    return _sdkSignatureSecret;\n}\n\n")
				.Append("+ (void)setSdkSignatureSecret:(NSString *)sdkSignatureSecret {\n    _sdkSignatureSecret = [sdkSignatureSecret copy];\n}\n\n")

				.Append("+ (NSInteger)logLevel {\n    return _logLevel;\n}\n\n")
				.Append("+ (void)setLogLevel:(NSInteger)logLevel {\n    _logLevel = logLevel;\n}\n\n")

				.Append("+ (NSString *)appScheme {\n    return _appScheme;\n}\n\n")
				.Append("+ (void)setAppScheme:(NSString *)appScheme {\n    _appScheme = [appScheme copy];\n}\n\n")

				.Append("+ (NSInteger)sessionTimeoutSeconds {\n    return _sessionTimeoutSeconds;\n}\n\n")
				.Append("+ (void)setSessionTimeoutSeconds:(NSInteger)sessionTimeoutSeconds {\n    _sessionTimeoutSeconds = sessionTimeoutSeconds;\n}\n\n")

				 .Append("+ (BOOL)autoStartTrackingEnabled {\n    return _autoStartTrackingEnabled;\n}\n\n")
				.Append("+ (void)setAutoStartTrackingEnabled:(BOOL)autoStartTrackingEnabled {\n    _autoStartTrackingEnabled = autoStartTrackingEnabled;\n}\n\n")

				.Append("+ (BOOL)userInfoHashEnabled {\n    return _userInfoHashEnabled;\n}\n\n")
				.Append("+ (void)setUserInfoHashEnabled:(BOOL)userInfoHashEnabled {\n    _userInfoHashEnabled = userInfoHashEnabled;\n}\n\n")

				.Append("+ (BOOL)trackAirbridgeLinkOnly {\n    return _trackAirbridgeLinkOnly;\n}\n\n")
				.Append("+ (void)setTrackAirbridgeLinkOnly:(BOOL)trackAirbridgeLinkOnly {\n    _trackAirbridgeLinkOnly = trackAirbridgeLinkOnly;\n}\n\n")

				.Append("+ (BOOL)facebookDeferredAppLinkEnabled {\n    return _facebookDeferredAppLinkEnabled;\n}\n\n")
				.Append("+ (void)setFacebookDeferredAppLinkEnabled:(BOOL)facebookDeferredAppLinkEnabled {\n    _facebookDeferredAppLinkEnabled = facebookDeferredAppLinkEnabled;\n}\n\n")

				.Append("+ (NSInteger)trackingAuthorizeTimeoutSeconds {\n    return _trackingAuthorizeTimeoutSeconds;\n}\n\n")
				.Append("+ (void)setTrackingAuthorizeTimeoutSeconds:(NSInteger)trackingAuthorizeTimeoutSeconds {\n    _trackingAuthorizeTimeoutSeconds = trackingAuthorizeTimeoutSeconds;\n}\n\n")

				.Append("+ (BOOL)trackInSessionLifeCycleEventEnabled {\n    return _trackInSessionLifeCycleEventEnabled;\n}\n\n")
				.Append("+ (void)setTrackInSessionLifeCycleEventEnabled:(BOOL)trackInSessionLifeCycleEventEnabled {\n    _trackInSessionLifeCycleEventEnabled = trackInSessionLifeCycleEventEnabled;\n}\n\n")

				.Append("+ (BOOL)pauseEventTransmitOnBackgroundEnabled {\n    return _pauseEventTransmitOnBackgroundEnabled;\n}\n\n")
				.Append("+ (void)setPauseEventTransmitOnBackgroundEnabled:(BOOL)pauseEventTransmitOnBackgroundEnabled {\n    _pauseEventTransmitOnBackgroundEnabled = pauseEventTransmitOnBackgroundEnabled;\n}\n\n")

				.Append("+ (BOOL)clearEventBufferOnInitializeEnabled {\n    return _clearEventBufferOnInitializeEnabled;\n}\n\n")
				.Append("+ (void)setClearEventBufferOnInitializeEnabled:(BOOL)clearEventBufferOnInitializeEnabled {\n    _clearEventBufferOnInitializeEnabled = clearEventBufferOnInitializeEnabled;\n}\n\n")

				.Append("+ (BOOL)sdkEnabled {\n    return _sdkEnabled;\n}\n\n")
				.Append("+ (void)setSdkEnabled:(BOOL)sdkEnabled {\n    _sdkEnabled = sdkEnabled;\n}\n\n")

				.Append("+ (NSInteger)eventBufferCountLimitInGibibyte {\n    return _eventBufferCountLimitInGibibyte;\n}\n\n")
				.Append("+ (void)setEventBufferCountLimitInGibibyte:(NSInteger)eventBufferCountLimitInGibibyte {\n    _eventBufferCountLimitInGibibyte = eventBufferCountLimitInGibibyte;\n}\n\n")

				.Append("+ (NSInteger)eventBufferSizeLimitInGibibyte {\n    return _eventBufferSizeLimitInGibibyte;\n}\n\n")
				.Append("+ (void)setEventBufferSizeLimitInGibibyte:(NSInteger)eventBufferSizeLimitInGibibyte {\n    _eventBufferSizeLimitInGibibyte = eventBufferSizeLimitInGibibyte;\n}\n\n")

				.Append("+ (NSInteger)eventTransmitIntervalSeconds {\n    return _eventTransmitIntervalSeconds;\n}\n\n")
				.Append("+ (void)setEventTransmitIntervalSeconds:(NSInteger)eventTransmitIntervalSeconds {\n    _eventTransmitIntervalSeconds = eventTransmitIntervalSeconds;\n}\n\n")

				.Append("+ (BOOL)isHandleAirbridgeDeeplinkOnly {\n    return _isHandleAirbridgeDeeplinkOnly;\n}\n\n")
				.Append("+ (void)setIsHandleAirbridgeDeeplinkOnly:(BOOL)isHandleAirbridgeDeeplinkOnly {\n    _isHandleAirbridgeDeeplinkOnly = isHandleAirbridgeDeeplinkOnly;\n}\n\n")
				
				.Append("+ (NSString *)inAppPurchaseEnvironment {\n    return _inAppPurchaseEnvironment;\n}\n\n")
				.Append("+ (void)setInAppPurchaseEnvironment:(NSString *)inAppPurchaseEnvironment {\n    _inAppPurchaseEnvironment = [inAppPurchaseEnvironment copy];\n}\n\n")
				
				.Append("+ (BOOL)collectTCFDataEnabled {\n    return _collectTCFDataEnabled;\n}\n\n")
				.Append("+ (void)setCollectTCFDataEnabled:(BOOL)collectTCFDataEnabled {\n    _collectTCFDataEnabled = collectTCFDataEnabled;\n}\n\n")
				
			   // End of implementation
			  .Append("@end\n").ToString();

		File.WriteAllText(implementationPath, implementationContent);

		Console.WriteLine("Updated iOS Airbridge settings with getters and setters.");
	}
	catch (Exception e)
	{
		Console.WriteLine("Something broken while updating iOS Airbridge settings files: {0}", e.Message);
	}
}

#endregion
}

public static class ConfigHierarchyExtensions
{
	public static bool GetInt64(this ConfigHierarchy configHierarchy, string sectionName, string keyName, out long value)
	{
		double doubleValue = 0;
		bool toReturn = configHierarchy.TryGetValue(sectionName, keyName, out doubleValue);
		value = (long) doubleValue;
		return toReturn;
	}
	
	public static bool GetAirbridgeLogLevel(this ConfigHierarchy configHierarchy, string sectionName, string keyName, out int value)
	{
		string stringValue;
		bool toReturn = configHierarchy.GetString(sectionName, keyName, out stringValue);
        switch (stringValue)
        {
            case "Debug"    : value = 0; break;
            case "Info"     : value = 1; break;
            case "Warning"  : value = 2; break;
            case "Error"    : value = 3; break;
            case "Fault"    : value = 4; break;

			default         : value = 2; break;	// default is "Warning"
        }
		return toReturn;
	}
	
	public static bool GetAirbridgeInAppPurchaseEnvironment(this ConfigHierarchy configHierarchy, string sectionName, string keyName, out string value)
	{
		string stringValue;
		bool toReturn = configHierarchy.GetString(sectionName, keyName, out stringValue);
		switch (stringValue)
		{
			case "Sandbox"		: value = "sandbox";	break;
			case "Production"	: value = "production";	break;
			
			default				: value = "production";	break;	// default is "Production"
		}
		return toReturn;
	}
}
