// Copyright © 2023 ab180. All rights reserved.

// Suppress:
//		ReSharper disable CppMemberFunctionMayBeStatic

#include "Airbridge/Data/AirbridgeInAppPurchase.h"

#if PLATFORM_ANDROID
#include "Platforms/Android/Util/AirbridgeJNI.h"
#elif PLATFORM_IOS
#import "Platforms/iOS/Native/AirbridgeInAppPurchaseBridge.h"
#endif

#if PLATFORM_ANDROID

UAirbridgeInAppPurchase* UAirbridgeInAppPurchase::CreateObject(jobject JavaObject)
{
	UAirbridgeInAppPurchase* AirbridgeInAppPurchase  = NewObject<UAirbridgeInAppPurchase>(GetTransientPackage(), StaticClass());
	AirbridgeInAppPurchase->Env = FAndroidApplication::GetJavaEnv();
	AirbridgeInAppPurchase->JavaObject = JavaObject;
	return AirbridgeInAppPurchase;
}

#elif PLATFORM_IOS

UAirbridgeInAppPurchase* UAirbridgeInAppPurchase::CreateObject(id inAppPurchaseBridge)
{
    UAirbridgeInAppPurchase* AirbridgeInAppPurchase  = NewObject<UAirbridgeInAppPurchase>(GetTransientPackage(), StaticClass());
    AirbridgeInAppPurchase->InAppPurchaseBridge = inAppPurchaseBridge;
    return AirbridgeInAppPurchase;
}

#endif

void UAirbridgeInAppPurchase::SetSemanticAttributes(const UAirbridgeMap* SemanticAttributes)
{
#if PLATFORM_ANDROID
	jmethodID Method = GetMethodID("setSemanticAttributes", "(Ljava/util/Map;)V");
	if (Method == nullptr) { return; }

	jobject jSemanticAttributes = (jobject) SemanticAttributes->CreateJavaObject();
	Env->CallVoidMethod(JavaObject, Method, jSemanticAttributes);
    
#elif PLATFORM_IOS
    NSDictionary *semanticDictionary = SemanticAttributes->ConvertToNSDictionary();
    [((AirbridgeInAppPurchaseBridge *)InAppPurchaseBridge) setSemanticAttributes:semanticDictionary];
    
#endif
}

void UAirbridgeInAppPurchase::SetCustomAttributes(const UAirbridgeMap* CustomAttributes)
{
#if PLATFORM_ANDROID
	jmethodID Method = GetMethodID("setCustomAttributes", "(Ljava/util/Map;)V");
	if (Method == nullptr) { return; }

	jobject jCustomAttributes = (jobject) CustomAttributes->CreateJavaObject();
	Env->CallVoidMethod(JavaObject, Method, jCustomAttributes);
    
#elif PLATFORM_IOS
    NSDictionary *customDictionary = CustomAttributes->ConvertToNSDictionary();
    [((AirbridgeInAppPurchaseBridge *)InAppPurchaseBridge) setCustomAttributes:customDictionary];
    
#endif
}

FString UAirbridgeInAppPurchase::GetPurchase() const
{
	FString ToReturn;
#if PLATFORM_ANDROID
	jmethodID Method = GetMethodID("getPurchase", "()Lco/ab180/airbridge/common/Purchase;");
	if (Method == nullptr) { return ToReturn; }

	jobject Purchase = Env->CallObjectMethod(JavaObject, Method);
	if (Purchase == nullptr) { return ToReturn; }

	jclass PurchaseClass = Env->GetObjectClass(Purchase);
	jmethodID GetOriginalJsonMethod = Env->GetMethodID(PurchaseClass, "getOriginalJson", "()Ljava/lang/String;");
	if (GetOriginalJsonMethod == nullptr) { return ToReturn; }
	
	jstring jOriginalJson = (jstring)Env->CallObjectMethod(Purchase, GetOriginalJsonMethod);
	AirbridgeJNI::ConvertString(Env, jOriginalJson, ToReturn);
#endif
	return ToReturn;
}

FString UAirbridgeInAppPurchase::GetTransaction() const
{
	FString ToReturn;
#if PLATFORM_IOS
    NSString *transactionString = [((AirbridgeInAppPurchaseBridge *)InAppPurchaseBridge) fetchTransactionString];
    ToReturn = FString(UTF8_TO_TCHAR(transactionString.UTF8String));
#endif
	return ToReturn;
}

#if PLATFORM_ANDROID

jmethodID UAirbridgeInAppPurchase::GetMethodID(const char* Name, const char* Sig) const
{
	if (Env == nullptr || JavaObject == nullptr) { return nullptr; }

	jclass AirbridgeInAppPurchaseClass = Env->FindClass("co/ab180/airbridge/AirbridgeInAppPurchase");
	if (AirbridgeInAppPurchaseClass == nullptr) { return nullptr; }

	return Env->GetMethodID(AirbridgeInAppPurchaseClass, Name, Sig);
}

#endif
