// Copyright © 2023 ab180. All rights reserved.

#include "Blueprints/AirbridgeBlueprint.h"

UAirbridgeBlueprint::UAirbridgeBlueprint(const FObjectInitializer& ObjectInitializer)
	: Super(ObjectInitializer)
{
}

#pragma region Core

void UAirbridgeBlueprint::EnableSDK()
{
	FAirbridge::EnableSDK();
}

void UAirbridgeBlueprint::DisableSDK()
{
	FAirbridge::DisableSDK();
}

bool UAirbridgeBlueprint::IsSDKEnabled()
{
	return FAirbridge::IsSDKEnabled();
}

#pragma endregion

#pragma region Privacy

void UAirbridgeBlueprint::StartTracking()
{
	FAirbridge::StartTracking();
}

void UAirbridgeBlueprint::StopTracking()
{
	FAirbridge::StopTracking();
}

bool UAirbridgeBlueprint::IsTrackingEnabled()
{
	return FAirbridge::IsTrackingEnabled();
}

#pragma endregion

#pragma region Deeplink

void UAirbridgeBlueprint::SetOnDeeplinkReceived()
{
	FAirbridge::SetOnDeeplinkReceived(nullptr);
}

#pragma endregion

#pragma region Data Collection

void UAirbridgeBlueprint::SetUserID(const FString& Id)
{
	FAirbridge::SetUserID(Id);
}

void UAirbridgeBlueprint::ClearUserID()
{
	FAirbridge::ClearUserID();
}

void UAirbridgeBlueprint::SetUserEmail(const FString& Email)
{
	FAirbridge::SetUserEmail(Email);
}

void UAirbridgeBlueprint::ClearUserEmail()
{
	FAirbridge::ClearUserEmail();
}

void UAirbridgeBlueprint::SetUserPhone(const FString& Phone)
{
	FAirbridge::SetUserPhone(Phone);
}

void UAirbridgeBlueprint::ClearUserPhone()
{
	FAirbridge::ClearUserPhone();
}

void UAirbridgeBlueprint::SetUserIntAttribute(const FString& Key, const int32 Value)
{
	FAirbridge::SetUserAttribute(Key, Value);
}

void UAirbridgeBlueprint::SetUserLongAttribute(const FString& Key, const int64 Value)
{
	FAirbridge::SetUserAttribute(Key, Value);
}

void UAirbridgeBlueprint::SetUserFloatAttribute(const FString& Key, const float Value)
{
	FAirbridge::SetUserAttribute(Key, Value);
}

void UAirbridgeBlueprint::SetUserBooleanAttribute(const FString& Key, const bool Value)
{
	FAirbridge::SetUserAttribute(Key, Value);
}

void UAirbridgeBlueprint::SetUserStringAttribute(const FString& Key, const FString& Value)
{
	FAirbridge::SetUserAttribute(Key, Value);
}

void UAirbridgeBlueprint::RemoveUserAttribute(const FString& Key)
{
	FAirbridge::RemoveUserAttribute(Key);
}

void UAirbridgeBlueprint::ClearUserAttributes()
{
	FAirbridge::ClearUserAttributes();
}

void UAirbridgeBlueprint::SetUserAlias(const FString& Key, const FString& Value)
{
	FAirbridge::SetUserAlias(Key, Value);
}

void UAirbridgeBlueprint::RemoveUserAlias(const FString& Key)
{
	FAirbridge::RemoveUserAlias(Key);
}

void UAirbridgeBlueprint::ClearUserAlias()
{
	FAirbridge::ClearUserAlias();
}

void UAirbridgeBlueprint::ClearUser()
{
	FAirbridge::ClearUser();
}

void UAirbridgeBlueprint::SetDeviceAlias(const FString& Key, const FString& Value)
{
	FAirbridge::SetDeviceAlias(Key, Value);
}

void UAirbridgeBlueprint::RemoveDeviceAlias(const FString& Key)
{
	FAirbridge::RemoveDeviceAlias(Key);
}

void UAirbridgeBlueprint::ClearDeviceAlias()
{
	FAirbridge::ClearDeviceAlias();
}

void UAirbridgeBlueprint::RegisterPushToken(const FString& Token)
{
	FAirbridge::RegisterPushToken(Token);
}

#pragma endregion

#pragma region Event

void UAirbridgeBlueprint::TrackEvent(
	const FString& EventCategory,
	const UAirbridgeMap* SemanticAttributes,
	const UAirbridgeMap* CustomAttributes
)
{
	FAirbridge::TrackEvent(EventCategory, SemanticAttributes, CustomAttributes);
}

#pragma endregion

#pragma region Placement

void UAirbridgeBlueprint::Click(const FString& TrackingLink, FAirbridgeVoidParamFunction OnSuccess, FAirbridgeStringParamFunction OnFailure)
{
	FAirbridge::Click(
		TrackingLink,
		[OnSuccess]()
		{
			if (OnSuccess.IsBound()) OnSuccess.Execute();
		},
		[OnFailure](const FString& Param)
		{
			if (OnFailure.IsBound()) OnFailure.Execute(Param);
		}
	);
}

void UAirbridgeBlueprint::Impression(const FString& TrackingLink, FAirbridgeVoidParamFunction OnSuccess, FAirbridgeStringParamFunction OnFailure)
{
	FAirbridge::Impression(
		TrackingLink,
		[OnSuccess]()
		{
			if (OnSuccess.IsBound()) OnSuccess.Execute();
		},
		[OnFailure](const FString& Param)
		{
			if (OnFailure.IsBound()) OnFailure.Execute(Param);
		}
	);
}

#pragma endregion

#pragma region Publication

void UAirbridgeBlueprint::FetchDeviceUUID(FAirbridgeStringParamFunction OnSuccess, FAirbridgeStringParamFunction OnFailure)
{
	FAirbridge::FetchDeviceUUID(
		[OnSuccess](const FString& Param)
		{
			if (OnSuccess.IsBound()) OnSuccess.Execute(Param);
		},
		[OnFailure](const FString& Param)
		{
			if (OnFailure.IsBound()) OnFailure.Execute(Param);
		}
	);
}

void UAirbridgeBlueprint::FetchAirbridgeGeneratedUUID(FAirbridgeStringParamFunction OnSuccess, FAirbridgeStringParamFunction OnFailure)
{
	FAirbridge::FetchAirbridgeGeneratedUUID(
		[OnSuccess](const FString& Param)
		{
			if (OnSuccess.IsBound()) OnSuccess.Execute(Param);
		},
		[OnFailure](const FString& Param)
		{
			if (OnFailure.IsBound()) OnFailure.Execute(Param);
		}
	);
}

void UAirbridgeBlueprint::SetOnAttributionReceived()
{
	FAirbridge::SetOnAttributionReceived(nullptr);
}

void UAirbridgeBlueprint::CreateTrackingLink(
	const FString& Channel,
	const UAirbridgeMap* Option,
	FAirbridgeTrackingLinkParamFunction OnSuccess,
	FAirbridgeStringParamFunction OnFailure
)
{
	FAirbridge::CreateTrackingLink(
		Channel,
		Option,
		[OnSuccess](const FAirbridgeTrackingLink& Param)
		{
			if (OnSuccess.IsBound()) OnSuccess.Execute(Param);
		},
		[OnFailure](const FString& Param)
		{
			if (OnFailure.IsBound()) OnFailure.Execute(Param);
		}
	);
}

#pragma endregion

#pragma region Hybrid

FString UAirbridgeBlueprint::CreateWebInterfaceScript(const FString& WebToken, const FString& PostMessageScript)
{
	return FAirbridge::CreateWebInterfaceScript(WebToken, PostMessageScript);
}

void UAirbridgeBlueprint::HandleWebInterfaceCommand(const FString& Command)
{
	FAirbridge::HandleWebInterfaceCommand(Command);
}

#pragma endregion

#pragma region IAP

void UAirbridgeBlueprint::StartInAppPurchaseTracking()
{
	FAirbridge::StartInAppPurchaseTracking();
}

void UAirbridgeBlueprint::StopInAppPurchaseTracking()
{
	FAirbridge::StopInAppPurchaseTracking();
}

bool UAirbridgeBlueprint::IsInAppPurchaseTrackingEnabled()
{
	return FAirbridge::IsInAppPurchaseTrackingEnabled();
}

#pragma endregion

bool UAirbridgeBlueprint::IsUninstallTrackingNotification(const TMap<FString, FString>& Data)
{
	return FAirbridge::IsUninstallTrackingNotification(Data);
}
