// Copyright © 2023 ab180. All rights reserved.

#pragma once

#include "CoreMinimal.h"
#include "Components/ActorComponent.h"
#include "AirbridgeCallbacks.generated.h"

DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnDeeplinkReceived, const FString&, Url);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnAttributionReceived, const FAirbridgeBlueprintMap&, AttributionResult);
DECLARE_DYNAMIC_MULTICAST_DELEGATE_OneParam(FOnInAppPurchaseReceived, UAirbridgeInAppPurchase*, InAppPurchase);

UCLASS( ClassGroup=("Airbridge"), meta=(BlueprintSpawnableComponent) )
class AIRBRIDGEUNREAL_API UAirbridgeCallbacks : public UActorComponent
{
	GENERATED_BODY()

public:	
	UAirbridgeCallbacks(const FObjectInitializer& ObjectInitializer);
	
	UPROPERTY(BlueprintAssignable, BlueprintCallable, Category = "Airbridge",
		meta=(ToolTip="This event is triggered when a deeplink is received."))
	FOnDeeplinkReceived OnDeeplinkReceived;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, Category = "Airbridge",
		meta=(ToolTip="This event is triggered when an attribution result data is received."))
	FOnAttributionReceived OnAttributionReceived;

	UPROPERTY(BlueprintAssignable, BlueprintCallable, Category = "Airbridge",
		meta=(ToolTip="This event is triggered when in-app purchase data is received."))
	FOnInAppPurchaseReceived OnInAppPurchaseReceived;
};